*****************************************************
****Chapter 10: Dichotomous or binary responses *****
*****************************************************


*** 10.2 Single level models for dichotomous responses

** 10.2.1 Generalized linear model formulation

use http://www.stata-press.com/data/mlmus3/womenlf, clear

recode workstat 2=1
logit workstat husbinc chilpres 
logit workstat husbinc chilpres, or

predict prob, p

twoway (line prob husbinc if chilpres==0, sort)         ///
 (line prob husbinc if chilpres==1, sort lpatt(dash)), ///
 legend(order(1 "No child" 2 "Child"))                  ///
 xtitle("Husband's income/$1000")                     ///
 ytitle("Probability that wife works")

twoway (function y=invlogit(_b[husbinc]*x+_b[_cons]), range(-100 100))    ///
 (function y=invlogit(_b[husbinc]*x+_b[chilpres]+_b[_cons]),              ///
 range(-100 100) lpatt(dash)),                                           ///
 xtitle("Husband's income/$1000") ytitle("Probability that wife works") ///
 legend(order(1 "No child" 2 "Child")) xline(1) xline(45)

glm workstat husbinc chilpres, link(logit) family(binomial)

** 10.2.2 Latent-response formulation
probit workstat husbinc chilpres

twoway (function y=invlogit(1.3358-0.0423*x), range(-100 100))           ///
 (function y=norm(0.7982-0.0242*x), range(-100 100) lpatt(dash)),        ///
 xtitle("Husband's income / $1000") ytitle("Probability that wife works") ///
 legend(order(1 "Logit link" 2 "Probit link")) xline(1) xline(45)

*** 10.3 Which treatment is best for toenail infection?

use http://www.stata-press.com/data/mlmus3/toenail, clear

*** 10.4 Longitudinal data structure

xtset patient visit
xtdescribe if outcome<.


*** 10.5 Population-averaged or marginal probabilities

label define tr 0 "Itraconazole" 1 "Terbinafine"
label values treatment tr

graph bar (mean) proportion= outcome, over(visit) by(treatment) ///
   ytitle(Proportion with onycholysis)

egen prop = mean(outcome), by(treatment visit)
egen mn_month = mean(month), by(treatment visit)

twoway line prop mn_month, by(treatment) sort ///
  xtitle(Time in months) ytitle(Probability with onycholysis)

generate trt_month = treatment*month
logit outcome treatment month trt_month, or vce(cluster patient)
predict prob, pr

twoway (line prop mn_month, sort) (line prob month, sort lpatt(dash) ),          ///
   by(treatment) legend(order(1 "Observed proportions" 2 "Fitted probabilities")) ///
   xtitle(Time in months) ytitle(Probability of onycholysis)


*** 10.7 Estimation of random-intercept logistic models

** 10.7.1 Using xtlogit
quietly xtset patient
xtlogit outcome treatment month trt_month, intpoints(30)
xtlogit, or

** 10.7.2 Using xtmelogit
xtmelogit outcome treatment month trt_month || patient:, intpoints(30)
estimates store xtmelogit

** 10.7.3 Using gllamm
gllamm outcome treatment month trt_month, i(patient) link(logit) family(binomial) ///
 nip(30) adapt
estimates store gllamm
*gllamm, eform


*** 10.8 Subject-specific vs. population averaged relationships

display (invlogit(1) + invlogit(2))/2
display invlogit((1+2)/1)


*** 10.9 Measures of dependence and heterogeneity

** 10.9.2 Median odds ratio

display exp(sqrt(2*16.084107)*invnorm(3/4))

** 10.9.3 Measures of association for observed responses...

* install xtrho: use findit xtrho, select st0031, then install


/*
* or use these commands:
net from http://www.stata-journal.com/software/sj3-1
net install st0031
*/

quietly xtset patient
quietly: xtlogit outcome treatment month trt_month, re intpoints(30)
xtrho

*copy http://www.stata-press.com/data/mlmus3/ch10table.do ch10table.do


*** 10.10 Inference for random-intercept logistic models 

** 10.10.1 Tests and confidence intervals for odds ratios
lincom treatment + trt_month*20
lincom treatment + trt_month*20, or
*lincom -(treatment + trt_month*20), or
*lincom -(1.treatment + 1.treatment#c.month*20), or

*** 10.11 Maximum likelihood estimation

** 10.11.2 Some speed and acuracy considerations
* Laplace approximation
xtmelogit outcome treatment month trt_month || patient:, intpoints(1)


*** 10.12 Assigning values to random effects

** 10.12.1 Maximum "likelihood" estimation

estimates restore xtmelogit
predict offset, xb

statsby mlest=_b[_cons], by(patient) saving(ml, replace): logit outcome, offset(offset)

sort patient
merge m:1 patient using ml
drop _merge

** 10.12.2 Empirical Bayes prediction

estimates restore gllamm
gllapred eb, u


** 10.12.3 Empirical Bayes modal prediction

estimates restore xtmelogit
predict ebmodal, reffects

twoway (rspike mlest ebmodal ebm1 if visit==1)                          ///
  (scatter mlest  ebm1 if visit==1, msize(small) msym(th) mcol(black))  ///
  (scatter ebmodal ebm1 if visit==1, msize(small) msym(oh) mcol(black)) ///
  (function y=x, range(ebm1) lpatt(solid)),                             ///
  xtitle(Empirical Bayes prediction)                                    ///
  legend(order(2 "Maximum likelihood" 3 "Empirical Bayes modal"))

predict se2, reses

egen num0 = total(outcome==0), by(patient)
egen num1 = total(outcome==1), by(patient)
list patient num0 num1 ebm1 ebmodal ebs1 se2 if visit==1&patient<=12, noobs


*** 10.13 Different kinds of predicted probabilities

** 10.13.1 Predicted population-averaged or marginal probabilities

estimates restore gllamm
gllapred margprob, mu marginal

twoway (line prob month, sort) (line margprob month, sort lpatt(dash) ),      ///
   by(treatment) legend(order(1 "Ordinary logit" 2 "Random intercept logit")) ///
   xtitle(Time in months) ytitle(Fitted marginal probabilities of onycholysis)

** 10.13.2 Predicted subject-specific probabilities

* Predictions for hypothetical subjects: Conditional
generate zeta1=0
gllapred condprob0, mu us(zeta)
generate lower1 = -4
gllapred condprobm4, mu us(lower)
generate upper1 = 4
gllapred condprob4, mu us(upper)
replace lower1 = -2
gllapred condprobm2, mu us(lower)
replace upper1 = 2
gllapred condprob2, mu us(upper)

twoway (line prop mn_month, sort)                               ///
 (line margprob month, sort lpatt(dash))                        ///
 (line condprob0 month, sort lpatt(shortdash_dot))              ///
 (line condprob4 month, sort lpatt(shortdash))                  ///
 (line condprobm4 month, sort lpatt(shortdash))                 ///
 (line condprob2 month, sort lpatt(shortdash))                  ///
 (line condprobm2 month, sort lpatt(shortdash)) ,               ///
 by(treatment)                                                  ///
 legend(order(1 "Observed proportion" 2 "Marginal probability"  ///
    3 "Median probability" 4 "Conditional probabilities"))      ///
 xtitle(Time in months) ytitle(Probabilities of onycholysis)

* Predictions for the subjects in the sample: Posterior mean
gllapred cmu, mu

sort patient visit
list patient visit if patient==2|patient==15, sepby(patient) noobs
fillin patient visit
list patient visit _fillin if patient==2|patient==15, sepby(patient) noobs

egen trt = mean(treatment), by(patient)
replace treatment = trt if _fillin==1
drop mn_month
egen mn_month = mean(month), by(treatment visit)
replace month = mn_month if _fillin==1
replace trt_month = treatment*month
drop cmu

gllapred cmu, mu fsample

list patient visit _fillin cmu if patient==2|patient==15, sepby(patient) noobs

set seed 1234421
sort patient
generate rand = runiform() if visit==1
by treatment (rand), sort: generate randid = _n if rand<.
egen randomid = mean(randid), by(patient)

twoway (line cmu month, sort) ///
    (scatter cmu month if _fillin==1, mcol(black)) ///
    if randomid<=16&treatment==0,  by(patient, compact legend(off) ///
	l1title("Posterior mean probabilities"))
	
twoway (line cmu month, sort) ///
    (scatter cmu month if _fillin==1, mcol(black)) ///
    if randomid<=16&treatment==1,  by(patient, compact legend(off) ///
	l1title("Posterior mean probabilities"))

 
*** 10.14 Other approaches to clustered dichotomous data

** 10.14.1 Conditional logistic regression

clogit outcome month trt_month, group(patient) or

** 10.14.2 Generalized estimating equations (GEE)

quietly xtset patient
xtgee outcome treatment month trt_month, link(logit) ///
  family(binomial) corr(exchangeable) vce(robust) eform
  
estat wcorrelation, format(%4.3f)
