**********************************************
****Chapter 6: Marginal models *****
**********************************************


** 6.3.1 Unstructured covariance matrix

use http://www.stata-press.com/data/mlmus3/wagepan, clear

generate educt = educ - 12
generate yeart = year - 1980

xtmixed lwage black hisp union married exper yeart educt || nr:, ///
    noconstant residuals(unstructured, t(yeart)) nofetable nogroup mle
estimates store un

ssc install xtmixed_corr, replace

xtmixed_corr

** 6.3.2. Random-intercept or compound-symmetric/exchangeable structure


quietly xtmixed lwage black hisp union married exper yeart educt || nr:, noconstant ///
    residuals(exchangeable) mle
estimates store ri


** 6.3.3 Random-coefficient structure


xtmixed lwage black hisp union married exper yeart educt || nr: yeart, ///
   covariance(unstructured) nofetable nogroup mle
estimates store rc

xtmixed_corr

** 6.3.4 Autoregressive and exponential structures  

xtmixed lwage black hisp union married exper yeart educt || nr:, noconstant ///
      residuals(ar 1, t(yeart)) nofetable nogroup mle
estimates store ar1

xtmixed_corr

** 6.3.5. Moving average residual structure

xtmixed lwage black hisp union married exper yeart educt || nr:, noconstant ///
     residuals(ma 1, t(yeart)) nofetable nogroup mle 
estimates store ma1

xtmixed_corr

** 6.3.6 Banded and Toeplitz structures

xtmixed lwage black hisp union married exper yeart educt || nr:, noconstant ///
     residuals(banded 1, t(yeart)) nofetable nogroup  mle
estimates store ba1

xtmixed_corr

xtmixed lwage black hisp union married exper yeart educt || nr:, noconstant ///
     residuals(toeplitz 2, t(yeart)) nofetable nogroup mle
estimates store to2
	 
xtmixed_corr


*** 6.4 Hybrid and complex marginal models

** 6.4.1 Random effects and correlated level-1 residuals

xtmixed lwage black hisp union married exper yeart educt || ///
    nr:, residuals(ar 1, t(yeart)) nofetable nogroup mle 
estimates store ri_ar1

xtmixed_corr

** 6.4.2 Heteroskedastic level-1 residuals over occasions

xtmixed lwage black hisp union married exper yeart educt || nr: yeart, ///
    covariance(unstructured) residuals(independent, by(yeart)) nofetable nogroup mle 
estimates store rc_het

xtmixed_corr

** 6.4.3 Heteroskedastic level-1 residuals over groups

generate ethnic = black*1 + hisp*2

xtmixed lwage black hisp union married exper yeart educt || nr:, ///
    residuals(independent, by(ethnic)) nofetable nogroup mle
estimates store ri_het2

table ethnic, contents(min nr)
xtmixed_corr, at(nr=13)
xtmixed_corr, at(nr=383)
xtmixed_corr, at(nr=1142)

** 6.4.4 Different covariance matrices over groups
xtmixed lwage black hisp union married exper yeart educt || nr:, ///
    residuals(ar 1, t(yeart) by(ethnic)) nofetable nogroup mle

/*
xtmixed_corr, at(nr=13)
xtmixed_corr, at(nr=383)
xtmixed_corr, at(nr=1142)
*/
	
*** 6.5 Comparing the fit of marginal models

lrtest un ma1

estimates stats un ri rc ar1 ma1 ba1 to2 rc_het

estimates table un ri rc ar1 ma1 ba1 to2 rc_het, ///
  b(%4.3f) se(%4.3f) keep(lwage:)

*** 6.6 Generalized Estimating Equations (GEE)

quietly xtset nr yeart
xtgee lwage black hisp union married exper yeart educt, ///
   corr(ar 1) vce(robust)

/*
xtmixed lwage black hisp union married exper yeart educt || nr:, noconstant ///
   residuals(ar 1, t(yeart)) vce(robust)
*/

estat wcorrelation, format(%4.3f)

/*
xtset nr yeart
xtreg lwage black hisp union married exper yeart educt, pa ///
 corr(ar 1) vce(robust)
matrix list e(R)
*/

*** 6.7 Marginal moceling with few units and many occasions

** 6.7.3 Fitting marginal models for long panels in Stata

use http://www.stata-press.com/data/mlmus3/garrett, clear

generate left_org =  leftpow*organ

xtset country year

xtpcse gdp oild demand leftpow organ left_org

xtpcse gdp oild demand leftpow organ left_org, correlation(ar1)
